/*-------------------------------------------------------------------------
 *
 * pg_pltemplate_d.h
 *    Macro definitions for pg_pltemplate
 *
 * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_PLTEMPLATE_D_H
#define PG_PLTEMPLATE_D_H

#define PLTemplateRelationId 1136

#define Anum_pg_pltemplate_tmplname 1
#define Anum_pg_pltemplate_tmpltrusted 2
#define Anum_pg_pltemplate_tmpldbacreate 3
#define Anum_pg_pltemplate_tmplhandler 4
#define Anum_pg_pltemplate_tmplinline 5
#define Anum_pg_pltemplate_tmplvalidator 6
#define Anum_pg_pltemplate_tmpllibrary 7
#define Anum_pg_pltemplate_tmplacl 8

#define Natts_pg_pltemplate 8


#endif							/* PG_PLTEMPLATE_D_H */
